==> tecfy_core_package-1.2.8.tar.gz: lib/services/init.service.dart <==
part of tecfy_core_client_package;

class InitService {
  /// menu items
  static List<TecfyMenuItem>? menuItems;

  /// is initlized filed
  static bool isInitled = false;

  static Future<dynamic> initTest() async {
    print('start init');
    // TecfyCoreApp.packageInfo = PackageInfo(
    //     appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
    await ConfigService.init();
    print('start db');
    // await DatabaseService.init();
    print('start logs');
    await LogService.init();
    print('start account');
  }

  static Future<dynamic> initTestAfter(context) async {
    print('start init');
    // TecfyCoreApp.packageInfo = PackageInfo(
    //     appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
    await ConfigService.init();
    print('start db');

    print('start logs');
    //await LogService.init();
    print('start account');
    await AccountService.init(context, withRedirection: false);
    print('finish account');

    return null;
  }

  static bool isBusy = false;
  static Future<dynamic> init(context, {bool withRedirection = true}) async {
    try {
      if (isBusy) return;
      isBusy = true;
      // if (isInitled) return;
      SoundService.init();
      // try {
      //   TecfyCoreApp.packageInfo =
      //       await PackageInfo.fromPlatform().timeout(Duration(seconds: 1));
      // } catch (err) {
      //   TecfyCoreApp.packageInfo = PackageInfo(
      //       appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
      // }
      await ConfigService.init();
      if (kDebugMode) print('start db');
      await DatabaseService.init();
      if (kDebugMode) print('start logs');
      await LogService.init();
      if (kDebugMode) print('start account');
      isInitled = true;
      await AccountService.init(context, withRedirection: withRedirection);
      if (kDebugMode) print('finish account!');
      await TecfyCoreApp.initService?.preLoginInit(context);
      isBusy = false;
      return null;
    } catch (err, t) {
      print('XXXX init error $err $t');
    }
    isBusy = false;
  }

  static List<Map<String, dynamic>> transList = [];

  static Future<void> initCompanyTranslation() async {
    try {
      OperationResult res =
          await EntityService.getDataEx('companyTranslation', allRecords: true);
      //print(res.data);
      if (!res.success) {
        print('XXXXX Cannot get CompanyTranslate $res');
        return;
      }
      transList = res.data;
      res.data.forEach((element) {
        AppLocale._sentences![element['textValueFrom']] =
            element['textValueTo'];
      });
    } catch (err) {
      print('XXXX CompanyTranslate $err');
    }
  }

  static Future afterLoginInit() async {
    try {
      WidgetsFlutterBinding.ensureInitialized();
      await initMenu();
      await initCompanyTranslation();
      CoreThemeService.init();
      DatabaseService.init();
      NotificationAppService.init();
      MessageService.init();
      WebSocketService.init();
      //MessageService.getNewMessages();

      var company = LocalConfigService.company;
      if (company['logoId'] == null) {
        try {
          var branchs = await EntityService.getDataEx('branch')
              .timeout(Duration(seconds: 5));
          var branch = branchs.data
                  ?.firstWhere((b) => b['logoId'] != null, orElse: () => {}) ??
              {};
          if (kDebugMode) print('branch $branch');
          company['logoId'] = branch['logoId'];
          if (company['logoId'] != null) {
            LocalConfigService.company = jsonDecode(
                jsonEncode(company, toEncodable: EntityService.customEncode));
          }
        } catch (err) {}
      }

      TecfyBasicApp.apiService.httpGetDynamic('startup').then((startupResult) {
        try {
          if (TecfyBasicApp.isNewer(startupResult.lastVersion)) {
            if (kDebugMode) print('need update');
            AppNavigator.navigateTo(TecfyBasicApp.appGlobalkey.currentContext!,
                () => UpgradePage(isRequired: false));
            //TecfyCoreApp.canUpgrade = true;
            //AppLocale.languageStreamController.add(LocalConfigService.language);
          }
        } catch (e) {}
      });

      await TecfyCoreApp.initService?.afterLoginInit(null);
    } catch (err, t) {
      throw "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX afterLoginInit() Error: $err $t";
    }
  }

  static initMenu() async {
    try {
      while (ConnectionService.isOnlineMode &&
          ConnectionService.activeConnection == null) {
        await Future.delayed(Duration(milliseconds: 100));
      }
      var result = ConnectionService.activeConnection == null
          ? OperationResult(
              success: false,
              message: 'No Active Connection to get GetEntityCategories')
          : await TecfyBasicApp.apiService
              .httpGetDynamic('entity/GetEntityCategories');
      if (!result.success) {
        print(result);
        if (result.errorCodeString == 'ClientVersionNotSupported') {
          TecfyCoreApp.mustUpgrade = true;
          AppLocale.languageStreamController.add(LocalConfigService.language);
          return;
        }
        result.data =
            jsonDecode(ConfigService.getValueString('EntityCategory'));
        result.success = (result.data != null);
      }

      if (result.success) {
        ConfigService.setValueString('EntityCategory',
            jsonEncode(result.data, toEncodable: EntityService.customEncode));
        EntityService.categories = [];
        for (var cat in result.data) {
          EntityService.categories!.add(EntityCategory.fromMap(cat));
        }
        await EntityService.initEntities();
        int ci = 100;
        int i = 100;

        if (kDebugMode)
          print(
              'EntityService.categories length is ${EntityService.categories!.length}');

        menuItems = EntityService.categories!
            .where((e) => e.entities.any((ee) => ee.permission.canView))
            .map<TecfyMenuItem>((c) {
          var menu = TecfyMenuItem(
              titleEn: c.nameEn,
              title: c.nameAr,
              orderId: ci++,
              code: c.dbName,
              subMenuItems: c.entities
                  .where((ee) => ee.permission.canView)
                  .map((e) => TecfyMenuItem(
                      orderId: i++,
                      title: e.name,
                      titleEn: e.nameEn,
                      code: e.dbName,
                      isLocked: e.menuLocked,
                      onClick: (context) {
                        var isMobile = LayoutService.isMobile(context);
                        if (isMobile) Navigator.of(context).pop();
                        AppNavigator.navigateTo(
                            context, () => EntityListPage(e.dbName),
                            animationType: isMobile
                                ? AnimationTypes.slideLeft
                                : AnimationTypes.opacity);
                      }))
                  .toList());
          i = 100;
          return menu;
        }).toList();

        if ((menuItems?.length ?? 0) == 0) {
          print(
              'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
          print(
              'menuItems length ${menuItems?.length} after appling permissions');
        }
        menuItems?.add(TecfyMenuItem(
            orderId: ci++,
            title: 'التقارير',
            titleEn: 'Reports',
            code: 'report',
            onClick: (context) {
              if (LayoutService.isMobile(context)) Navigator.pop(context);
              AppNavigator.navigateTo(context, () => ReportListPage());
            }));

        // if (LocalConfigService.company['id'] == 'tecfypos') {
        //   //EntityService.entities?['company']?.entityType = EntityTypes.MenuItem;
        //   EntityService.entities?['company']?.permission =
        //       Permission.fullAccess();
        //   // menuItems?.add(
        //   //   MenuItem(
        //   //       orderId: ci++,
        //   //       title: 'Companies & Accounts',
        //   //       code: 'company',
        //   //       onClick: (context) {
        //   //         if (LayoutService.isMobile(context)) Navigator.pop(context);
        //   //         AppNavigator.navigateTo(
        //   //             context, () => EntityListPage('company'));
        //   //       }),
        //   // );
        // }
      } else {
        print('XXXXXX get entities failed : ${result.message}');
        menuItems = [
          TecfyMenuItem(
              orderId: 500,
              title: 'Settings',
              code: 'setting',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                // AppNavigator.navigateToNamed(context, '/setting');
                AppNavigator.showAppDialogEx(context, () => SettingPage(true),
                    size: DialogBoxSizes.large,
                    barrierDismissible: true,
                    title: AppLocale.translate('Settings'));
              }),
          TecfyMenuItem(
              orderId: 600,
              title: 'Server Connections',
              code: '_ServerConnections',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                // AppNavigator.navigateTo(
                //     context, () => EntityListPage('_ServerConnections'));
                AppNavigator.showAppDialogEx(context,
                    () => EntityListPage('_ServerConnections', isPopup: true),
                    size: DialogBoxSizes.large,
                    barrierDismissible: true,
                    closeButton: true);
              }),
          TecfyMenuItem(
              orderId: 1000,
              title: 'Logout',
              code: 'logout',
              onClick: (context) {
                AccountService.signout(context);
              }),
        ];
      }

      if (TecfyCoreApp.menuService != null) {
        menuItems = await TecfyCoreApp.menuService?.initMenuItems(menuItems!);
      }
      menuItems?.sort((a, b) => a.orderId - b.orderId);
    } catch (err, stake) {
      print('XXXXXXXXXX  initMenu : $err');
      print(stake);
      throw "init menu: $err";
    }
  }
}

==> tecfy_core_package-1.2.14.tar.gz: lib/services/init.service.dart <==
part of tecfy_core_client_package;

class InitService {
  /// menu items
  static List<TecfyMenuItem>? menuItems;

  /// is initlized filed
  static bool isInitled = false;

  static Future<dynamic> initTest() async {
    print('start init');
    // TecfyCoreApp.packageInfo = PackageInfo(
    //     appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
    await ConfigService.init();
    print('start db');
    // await DatabaseService.init();
    print('start logs');
    await LogService.init();
    print('start account');
  }

  static Future<dynamic> initTestAfter(context) async {
    print('start init');
    // TecfyCoreApp.packageInfo = PackageInfo(
    //     appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
    await ConfigService.init();
    print('start db');

    print('start logs');
    //await LogService.init();
    print('start account');
    await AccountService.init(context, withRedirection: false);
    print('finish account');

    return null;
  }

  static bool isBusy = false;
  static Future init(context, {bool withRedirection = true}) async {
    try {
      if (isBusy) return;
      isBusy = true;
      // if (isInitled) return;
      SoundService.init();
      // try {
      //   TecfyCoreApp.packageInfo =
      //       await PackageInfo.fromPlatform().timeout(Duration(seconds: 1));
      // } catch (err) {
      //   TecfyCoreApp.packageInfo = PackageInfo(
      //       appName: '', version: '0.0.0', buildNumber: '0', packageName: '');
      // }
      await ConfigService.init();
      if (kDebugMode) print('start db');
      await DatabaseService.init();
      if (kDebugMode) print('start logs');
      await LogService.init();
      if (kDebugMode) print('start account');
      isInitled = true;
      await AccountService.init(context, withRedirection: withRedirection);
      if (kDebugMode) print('finish account!');
      await TecfyCoreApp.initService?.preLoginInit(context);
      isBusy = false;
      return;
    } catch (err, t) {
      print('XXXX init error $err $t');
    }
    isBusy = false;
  }

  static List<Map<String, dynamic>> transList = [];

  static Future<void> initCompanyTranslation() async {
    try {
      OperationResult res =
          await EntityService.getDataEx('companyTranslation', allRecords: true);
      //print(res.data);
      if (!res.success) {
        print('XXXXX Cannot get CompanyTranslate $res');
        return;
      }
      transList = res.data;
      res.data.forEach((element) {
        AppLocale._sentences![element['textValueFrom']] =
            element['textValueTo'];
      });
    } catch (err) {
      print('XXXX CompanyTranslate $err');
    }
  }

  static Future afterLoginInit() async {
    try {
      WidgetsFlutterBinding.ensureInitialized();
      await initMenu();
      await initCompanyTranslation();
      CoreThemeService.init();
      DatabaseService.init();
      NotificationAppService.init();
      MessageService.init();
      WebSocketService.init();
      //MessageService.getNewMessages();

      var company = LocalConfigService.company;
      if (company['logoId'] == null) {
        try {
          var branchs = await EntityService.getDataEx('branch')
              .timeout(Duration(seconds: 5));
          var branch = branchs.data
                  ?.firstWhere((b) => b['logoId'] != null, orElse: () => {}) ??
              {};
          if (kDebugMode) print('branch $branch');
          company['logoId'] = branch['logoId'];
          if (company['logoId'] != null) {
            LocalConfigService.company = jsonDecode(
                jsonEncode(company, toEncodable: EntityService.customEncode));
          }
        } catch (err) {}
      }

      TecfyBasicApp.apiService.httpGetDynamic('startup').then((startupResult) {
        try {
          if (TecfyBasicApp.isNewer(startupResult.lastVersion)) {
            if (kDebugMode) print('need update');
            AppNavigator.navigateTo(TecfyBasicApp.appGlobalkey.currentContext!,
                () => UpgradePage(isRequired: false));
            //TecfyCoreApp.canUpgrade = true;
            //AppLocale.languageStreamController.add(LocalConfigService.language);
          }
        } catch (e) {}
      });

      await TecfyCoreApp.initService?.afterLoginInit(null);
    } catch (err, t) {
      throw "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX afterLoginInit() Error: $err $t";
    }
  }

  static initMenu() async {
    try {
      while (ConnectionService.isOnlineMode &&
          ConnectionService.activeConnection == null) {
        await Future.delayed(Duration(milliseconds: 100));
      }
      var result = ConnectionService.activeConnection == null
          ? OperationResult(
              success: false,
              message: 'No Active Connection to get GetEntityCategories')
          : await TecfyBasicApp.apiService
              .httpGetDynamic('entity/GetEntityCategories');
      if (!result.success) {
        print(result);
        if (result.errorCodeString == 'ClientVersionNotSupported') {
          TecfyCoreApp.mustUpgrade = true;
          AppLocale.languageStreamController.add(LocalConfigService.language);
          return;
        }
        result.data =
            jsonDecode(ConfigService.getValueString('EntityCategory'));
        result.success = (result.data != null);
      }

      if (result.success) {
        ConfigService.setValueString('EntityCategory',
            jsonEncode(result.data, toEncodable: EntityService.customEncode));
        EntityService.categories = [];
        for (var cat in result.data) {
          EntityService.categories!.add(EntityCategory.fromMap(cat));
        }
        await EntityService.initEntities();
        int ci = 100;
        int i = 100;

        if (kDebugMode)
          print(
              'EntityService.categories length is ${EntityService.categories!.length}');

        menuItems = EntityService.categories!
            .where((e) => e.entities.any((ee) => ee.permission.canView))
            .map<TecfyMenuItem>((c) {
          var menu = TecfyMenuItem(
              titleEn: c.nameEn,
              title: c.nameAr,
              orderId: ci++,
              code: c.dbName,
              subMenuItems: c.entities
                  .where((ee) => ee.permission.canView)
                  .map((e) => TecfyMenuItem(
                      orderId: i++,
                      title: e.name,
                      titleEn: e.nameEn,
                      code: e.dbName,
                      isLocked: e.menuLocked,
                      onClick: (context) {
                        var isMobile = LayoutService.isMobile(context);
                        if (isMobile) Navigator.of(context).pop();
                        AppNavigator.navigateTo(
                            context, () => EntityListPage(e.dbName),
                            animationType: isMobile
                                ? AnimationTypes.slideLeft
                                : AnimationTypes.opacity);
                      }))
                  .toList());
          i = 100;
          return menu;
        }).toList();

        if ((menuItems?.length ?? 0) == 0) {
          print(
              'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX');
          print(
              'menuItems length ${menuItems?.length} after appling permissions');
        }
        menuItems?.add(TecfyMenuItem(
            orderId: ci++,
            title: 'التقارير',
            titleEn: 'Reports',
            code: 'report',
            onClick: (context) {
              if (LayoutService.isMobile(context)) Navigator.pop(context);
              AppNavigator.navigateTo(context, () => ReportListPage());
            }));

        // if (LocalConfigService.company['id'] == 'tecfypos') {
        //   //EntityService.entities?['company']?.entityType = EntityTypes.MenuItem;
        //   EntityService.entities?['company']?.permission =
        //       Permission.fullAccess();
        //   // menuItems?.add(
        //   //   MenuItem(
        //   //       orderId: ci++,
        //   //       title: 'Companies & Accounts',
        //   //       code: 'company',
        //   //       onClick: (context) {
        //   //         if (LayoutService.isMobile(context)) Navigator.pop(context);
        //   //         AppNavigator.navigateTo(
        //   //             context, () => EntityListPage('company'));
        //   //       }),
        //   // );
        // }
      } else {
        print('XXXXXX get entities failed : ${result.message}');
        menuItems = [
          TecfyMenuItem(
              orderId: 500,
              title: 'Settings',
              code: 'setting',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                // AppNavigator.navigateToNamed(context, '/setting');
                AppNavigator.showAppDialogEx(context, () => SettingPage(true),
                    size: DialogBoxSizes.large,
                    barrierDismissible: true,
                    title: AppLocale.translate('Settings'));
              }),
          TecfyMenuItem(
              orderId: 600,
              title: 'Server Connections',
              code: '_ServerConnections',
              onClick: (context) {
                if (LayoutService.isMobile(context)) Navigator.pop(context);
                // AppNavigator.navigateTo(
                //     context, () => EntityListPage('_ServerConnections'));
                AppNavigator.showAppDialogEx(context,
                    () => EntityListPage('_ServerConnections', isPopup: true),
                    size: DialogBoxSizes.large,
                    barrierDismissible: true,
                    closeButton: true);
              }),
          TecfyMenuItem(
              orderId: 1000,
              title: 'Logout',
              code: 'logout',
              onClick: (context) {
                AccountService.signout(context);
              }),
        ];
      }

      if (TecfyCoreApp.menuService != null) {
        menuItems = await TecfyCoreApp.menuService?.initMenuItems(menuItems!);
      }
      menuItems?.sort((a, b) => a.orderId - b.orderId);
    } catch (err, stake) {
      print('XXXXXXXXXX  initMenu : $err');
      print(stake);
      throw "init menu: $err";
    }
  }
}
